/****************************************************************************
*
*	  Header Filename:  Molecule_Type.h
*	  Module Name:      Molecule_Type
*	  Description:      A Molecule Type that can exist in the Chemistry
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Molecule_Type_H
#define Molecule_Type_H

class   Molecule_Type;  // Type specifier used in Realised_Molecule_Type
class   Realised_Molecule_Type;
class   Network_Painter_Prot;
class   Interaction_Type;

//  Class declaration


class   Molecule_Type

//  A Molecule_Type that can exist in the Chemistry

{
    public:
//==== =Relationships with other classes ===================================================================

        class  List_Of_Int_Types_With_MolType_As_Product_Class

//          Class to maintain List of Interaction_types that have the Molecule_Type as a Product

        {
            private:
                vector<Interaction_Type*>   Vector__Int_Types_For_Product;  // List of Interaction_Types
                                                                            // for which Molecule_Type is a
                                                                            // Product

                vector<Interaction_Type*>::iterator   Iter_Int_Type_With_Mol_Type_As_Product;

            public:

                void    Add(Interaction_Type*   Int_Type);
                void    Clear();
                void    Remove(Interaction_Type*   Int_Type);
                Interaction_Type*  Get_Current_Item();
                bool    Iter_To_First();
                bool    Iter_To_Next();
        };
        List_Of_Int_Types_With_MolType_As_Product_Class    List_Of_Int_Types_With_MolType_As_Product;

        class  List_Of_Int_Types_With_MolType_As_Input_Class

//          Class to maintain List of Interaction_types that have the Molecule_Type as an Input (Reactant)

        {
            private:
                vector<Interaction_Type*>   Vector__Int_Types_For_Input;    // List of Interaction_Types
                                                                            // for which Molecule_Type is an
                                                                            // Input

                vector<Interaction_Type*>::iterator   Iter_Int_Type_With_Mol_Type_As_Input;

            public:

                void    Add(Interaction_Type*   Int_Type);
                void    Clear();
                void    Remove(Interaction_Type*   Int_Type);             
                Interaction_Type*  Get_Current_Item();
                bool    Iter_To_First();
                bool    Iter_To_Next();
        };
        List_Of_Int_Types_With_MolType_As_Input_Class    List_Of_Int_Types_With_MolType_As_Input;

        class  Mass_Tp   // Used for polymorphic Molecule Type constructors
        {
            public:
                Mass_Tp(int Value) {_Value = Value;};
                int  Get_Value() {return _Value;};
            private:
                int  _Value;
        };

        class  Potential_Energy_Tp  // Used for polymorphic Molecule Type constructors
        {
            public:
                Potential_Energy_Tp(double Value) {_Value = Value;};
                double  Get_Value() {return _Value;};
            private:
                double  _Value;
        };

        private:
          Realised_Molecule_Type*  Realised_Molecule_Type__Ptr;
//==========================================================================================================

//      Public Functions

    public:
        Molecule_Type ();
        Molecule_Type(Mass_Tp  MolType_Mass);
        Molecule_Type(Potential_Energy_Tp  MolType_Pot_Energy);
        Molecule_Type(Mass_Tp  MolType_Mass, Potential_Energy_Tp  MolType_Pot_Energy);

        void    Add_To_Chemistry();
        void    Clear_Lists_Of_Interaction_Types();
        int     Get_Identifier()  const;
        void    Set_Identifier(int  Identifier_Value);
        int     Get_Mass() const;
	    double  Get_Potential_Energy() const;
//        void    Set_Potential_Energy(double Pot_Energy);
	
        static int  Get_Max_Molecule_Mass() {return Max_Molecule_Mass;}
        static void Set_Max_Molecule_Mass(int Max_Mass) {Max_Molecule_Mass = Max_Mass;}

        void   Link_To_Realised_Molecule_Type  (Realised_Molecule_Type*  Realised_Mol_Type_Arg__Ptr);
        void    Unlink_From_R_Mol_Type();  

        Realised_Molecule_Type*     Get_Realised_Molecule_Type__Ptr() const;

        int  Get_Specification();
        void Set_Specification(int Specification_Value);
        static void Reset_Next_Molecule_Type_Identifier() {Next_Molecule_Type_Identifier = 0;}

//        bool Is_In_Reactor();

//----------------------------------------------------------------------------------------------------------
//      Private Class Data

    private:
        int Identifier;             // A unique identifier for the Molecule_type
        int Specification;          // The specification of the Molecule_Type
                                    // In Random Chemistry this is the same as
                                    // the Molecule_Type_Identifier
        int Mass;
        double  Potential_Energy;

        static int  Next_Molecule_Type_Identifier;  // Identifier for next Molecule_Type created
        static int  Max_Molecule_Mass;
        static double  Max_MolType_Pot_Energy;
   
//----------------------------------------------------------------------------------------------------------
//      Private Functions
    private:

        void    Link_Int_Type_With_This_Product(Interaction_Type*  Int_Type__Ptr);
        void    Initialise(int MolType_Mass, double MolType_Pot_Energy);
};

#endif  //Molecule_Type_H
