/****************************************************************************
*
*	  Module Filename: Network_Painter.cpp
*	  Module Name:     Network_Painter
*	  Description:
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

//#define  ENABLE_TRACE_FILE  

#include "globldef.h"
#include "Library_Headers.h"

//--------------  GUI Presentation Subsystem API ----------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Network Painter Subsystem  --------------------------------------------------------------------
#include "Network_Painter.h"
//---------------------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------------
void Network_Painter::Clear()

// Clear the Network Painter's data structures

{
    Clear_Network_Image();
}
//----------------------------------------------------------------------------------------------------------------
void Network_Painter::Decrease_Connection_Width()

//  Decrease the display width for Connections

{
    Display_Properties.Conn_Width_To_Strength_Ratio = Display_Properties.Conn_Width_To_Strength_Ratio/1000;

    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();
}
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Decrease_Node_Width()

//  Decrease the display width for Nodes

{
    Display_Properties.Node_Width_To_Strength_Ratio = Display_Properties.Node_Width_To_Strength_Ratio/2;

    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();
}

//----------------------------------------------------------------------------------------------------------
void Network_Painter::Draw_Construction(   int Reactant1_Node_Index,
                                           int Reactant2_Node_Index,
                                           int Product1_Node_Index,
                                           double Connection_Strength,
                                           int  Conn_Selection_State,
                                           bool Bool__Store_Network_Item)

//  Display a Construction on the Network_Painter's PaintBox

{
    int Connection_Width;
    Support::Connection_Display_Colours   Colours;
    Support::Determine_Connection_Width_And_Colour(Conn_Selection_State,
                                                   Connection_Strength,
                                                   Connection_Width,
                                                   &Colours,
                                                   this);

 if (Connection_Width >= 1)
    THEN

//      Draw the line for one part of the Construction

        Support::Join_Nodes(Reactant1_Node_Index,
                            Product1_Node_Index,
                            Connection_Width,
                            Colours.Main_Colour,
                            Colours.Reactant_End_Colour,
                            Colours.Product_End_Colour,
                            this,
                            Network_Drawing_Area_Id);

        //  Draw the line for the other part

        Support::Join_Nodes(Reactant2_Node_Index,
                            Product1_Node_Index,
                            Connection_Width,
                            Colours.Main_Colour,
                            Colours.Reactant_End_Colour,
                            Colours.Product_End_Colour,
                            this,
                            Network_Drawing_Area_Id);

//      If requested, create a Network_Display_Item
//      and store it in case the network needs to be displayed again

        if (Bool__Store_Network_Item == true)
        THEN
            Make_And_Store_Connection_Display_Item(Connection_Strength,
                                                   Conn_Selection_State,
                                                   Reactant1_Node_Index,
                                                   Reactant2_Node_Index, 
                                                   Product1_Node_Index,
                                                   0, // Product2_Node_Index
                                                   0); //Catalyst_Node_Index
        ENDIF
    ENDIF                                                   
}
//----------------------------------------------------------------------------------------------------------

void Network_Painter::Draw_Fission(int Reactant1_Node_Index,
                                   int Product1_Node_Index,
                                   int Product2_Node_Index,
                                   double Connection_Strength,
                                   int Conn_Selection_State,
                                   bool Bool__Store_Network_Item)

//  Display a Fission on the Network_Painter's PaintBox

{
    int Connection_Width;
    Support::Connection_Display_Colours   Colours;
    Support::Determine_Connection_Width_And_Colour(Conn_Selection_State,
                                                   Connection_Strength,
                                                   Connection_Width,
                                                   &Colours,
                                                   this);

 if (Connection_Width >= 1)
    THEN

//      Draw the line for one part of the Fission

        Support::Join_Nodes(Reactant1_Node_Index,
                            Product1_Node_Index,
                            Connection_Width,
                            Colours.Main_Colour,
                            Colours.Reactant_End_Colour,
                            Colours.Product_End_Colour,
                            this,
                            Network_Drawing_Area_Id);

        //  Draw the line for the other part

        Support::Join_Nodes(Reactant1_Node_Index,
                            Product2_Node_Index,
                            Connection_Width,
                            Colours.Main_Colour,
                            Colours.Reactant_End_Colour,
                            Colours.Product_End_Colour,
                            this,
                            Network_Drawing_Area_Id);

//      If requested, create a Network_Display_Item
//      and store it in case the network needs to be displayed again

        if (Bool__Store_Network_Item == true)
        THEN
            Make_And_Store_Connection_Display_Item(Connection_Strength,
                                                   Conn_Selection_State,
                                                   Reactant1_Node_Index,
                                                   0, // Reactant2_Node_Index,
                                                   Product1_Node_Index,
                                                   Product2_Node_Index,
                                                   0); //Catalyst_Node_Index
        ENDIF
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Draw_Molecule_Type(const int Molecule_Type_Identifier,

                                         const double Node_Strength,
                                         const Enum_Node_Selection_State  Selection_State,
                                         const bool Bool__Store_Network_Item)

//  Draw a Molecule_Type on the Network Painter's Paintbox. The size
//  of the circle indicates the number of Molecules
//  The colour depends on the requested Display Type

{

    Screen_Location   Screen_Location__For_Mol_Type
        = Support::Get_Screen_Location_For_Node(Molecule_Type_Identifier,this);
    int Display_Size = static_cast <int> (Node_Strength * Display_Properties.Node_Width_To_Strength_Ratio);


    GUI_Presentation_API::Set_Foreground_Colour(Network_Drawing_Area_Id, 
                                                Colour_Ids::Blue);

    
    GUI_Presentation_API::Draw_Rectangle(Network_Drawing_Area_Id,
                                         1, 
                                         Screen_Location__For_Mol_Type.X,
                                         Screen_Location__For_Mol_Type.Y,
                                         Display_Size,
                                         Display_Size);

//  If requested, create a Network_Display_Item
//  and store it in case the network needs to be displayed again

    if (Bool__Store_Network_Item == true)

    THEN

        CREATE_OBJECT(Network_Display_Item, Network_Display_Item* Network_Display_Item__Ptr = new Network_Display_Item);

        Network_Display_Item__Ptr->Network_Item_Type =   Network_Display_Item__Ptr->Molecule_Type;
        Network_Display_Item__Ptr->Node_Index =  Molecule_Type_Identifier;
        Network_Display_Item__Ptr->Node_Strength =  Node_Strength;
        Network_Display_Item__Ptr->Selection_State =  Selection_State;

        List_Of_Network_Display_Items.push_back(Network_Display_Item__Ptr);
    ENDIF

}


//----------------------------------------------------------------------------------------------------------

void    Network_Painter::Draw_Text_Output(int X_Pos, int Y_Pos, const char* Output_Text_Ptr,
                                          bool Store_Network_Item)

//  Draw the text on the Network Painter's Paintbox at the specified location

{
//  If requested, create a Network_Text_Item
//  and store it in case the network needs to be displayed again

    if (Store_Network_Item)
    THEN
        CREATE_OBJECT(Network_Text_Item, Network_Text_Item* Network_Text_Item__Ptr = new Network_Text_Item);
        Network_Text_Item__Ptr->XPos = X_Pos;
        Network_Text_Item__Ptr->YPos = Y_Pos;
        Network_Text_Item__Ptr->Text_Item = string(Output_Text_Ptr);

        
        List_Of_Network_Text_Items.push_back(Network_Text_Item__Ptr);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Draw_Transformation( int Reactant1_Node_Index,
                                           int Product1_Node_Index,
                                           double Connection_Strength,
                                           int Conn_Selection_State,
                                           bool Bool__Store_Network_Item)

//  Display a Transformation on the Network_Painter's PaintBox

{
    int Connection_Width;
    Support::Connection_Display_Colours   Colours;
    Support::Determine_Connection_Width_And_Colour(Conn_Selection_State,
                                                   Connection_Strength,
                                                   Connection_Width,
                                                   &Colours,

                                                   this);

 if (Connection_Width >= 1)
    THEN

//      Draw the line for the Transformation

        Support::Join_Nodes(Reactant1_Node_Index,
                            Product1_Node_Index,
                            Connection_Width,
                            Colours.Main_Colour,
                            Colours.Reactant_End_Colour,
                            Colours.Product_End_Colour,
                            this,
                            Network_Drawing_Area_Id);




//      If requested, create a Network_Display_Item
//      and store it in case the network needs to be displayed again

        if (Bool__Store_Network_Item == true)
        THEN
            Make_And_Store_Connection_Display_Item(Connection_Strength,
                                                   Conn_Selection_State,
                                                   Reactant1_Node_Index,
                                                   0,  // Reactant2_Node_Index
                                                   Product1_Node_Index,
                                                   0,  // Product2_Node_Index
                                                   0); // Catalyst_Node_Index
        ENDIF
    ENDIF   
}

//----------------------------------------------------------------------------------------------------------

Network_Painter::Network_Painter(int     NumOfNodesPerDisplayRow,
                                 int     SpaceInPixelsBetweenNodes,
                                 double   Typical_Connection_Strength,
                                 double   Typical_Node_Strength,
                           //      void*   TPaintBox__NetworkDisplayPtr,
                                 bool    Display_All_Connections_Ind,
                                 Widget_Ids::Drawing_Area_Id  Draw_Area_Id_Arg)


//  Constructor for Network_Painter.

{
//  Setup Display Parameters, including the Colour Map for Node selection states

    Display_Properties.Num_Of_Nodes_Per_Display_Row = NumOfNodesPerDisplayRow;
    Display_Properties.Distance_In_Pixels_Between_Nodes = SpaceInPixelsBetweenNodes;

    Display_Properties.Conn_Width_To_Strength_Ratio = 1/Typical_Connection_Strength;
    Display_Properties.Node_Width_To_Strength_Ratio  = 1/Typical_Node_Strength;
    Display_Properties.Display_Unselected_Conns_Flag = Display_All_Connections_Ind;
    Display_Properties.Main_Display_Colour = Colour_Ids::Black;
    Display_Properties.Reactant_End_Colour = Colour_Ids::Red;
    Display_Properties.Product_End_Colour = Colour_Ids::Blue;    


    Set_Node_Selection_Colour_Map(EOpt_Node_Selected_As_Reactant, Colour_Ids::Red);
    Set_Node_Selection_Colour_Map(EOpt_Node_Selected_As_Product, Colour_Ids::Blue);
    Set_Node_Selection_Colour_Map(EOpt_Node_Selected_As_Catalyst, Colour_Ids::LimeGreen);
    Set_Node_Selection_Colour_Map(EOpt_Node_Not_Selected, Colour_Ids::Black);

    Network_Drawing_Area_Id = Draw_Area_Id_Arg;

//  Link the Network_Painter to its PaintBox

//    TPaintBox__Network_Display_Ptr = static_cast <TPaintBox*> (TPaintBox__NetworkDisplayPtr);
                                     // This is necessary to avoid having to declare
                                     // TPaintBox in Network Network_Painter_Prot.h
                                     // A forward declaration there leads to an ambiguity

// Setup a Bitmap

//    TBitmap__Network_Display_Ptr = TBitmap::TBitmap_Create();

//  Setup parameters for the Bitmap

//    TBitmap__Network_Display_Ptr->Set_Height(1024);
//    TBitmap__Network_Display_Ptr->Set_Width(1024);
//    TBitmap__Network_Display_Ptr->Canvas()->Brush()->Set_Color(clWhite());

//  Initialise the Bitmap display area

//    TBitmap__Network_Display_Ptr->Canvas()->Rectangle(0,0,1024,1024);

//  Remember the initial position of the top of the PaintBox's PaintBox_Top =
//  This is necessary due to Lazarus bug 361, due to which the Caanvas does
//  not move with the PaintBox

//    PaintBox_Top_Initial = TPaintBox__Network_Display_Ptr->Get_Top();

//  Create the list of Network_Display_Items

//    CREATE_OBJECT(TList, TList__Of_Network_Disp_Items_Ptr = new TList);
}
//----------------------------------------------------------------------------------------------------------
/*
TBitmap* Network_Painter::Get_TBitmap_Network_Display_Ptr() const

//  Return a pointer to the TBitmap used by the Network Painter

{
    return (TBitmap__Network_Display_Ptr);
}
*/
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Increase_Connection_Width()

//  Increase the display width for connections

{
    Display_Properties.Conn_Width_To_Strength_Ratio = Display_Properties.Conn_Width_To_Strength_Ratio*1000;
//    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();
}
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Increase_Node_Width()

//  Increase the display Width for Nodes

{
    Display_Properties.Node_Width_To_Strength_Ratio = Display_Properties.Node_Width_To_Strength_Ratio*2;

//    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();
}
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Make_And_Store_Connection_Display_Item
                                            (const double&   Connection_Strength,
                                             const int      Conn_Selection_State,
                                             const int&     Reactant1_Node_Index,
                                             const int&     Reactant2_Node_Index,
                                             const int&     Product1_Node_Index,
                                             const int&     Product2_Node_Index,
                                             const int&     Catalyst_Node_Index)

//  Store a Network Display Item for a Connection. Can be used if the
//  Connection has to be redrawn

{
    CREATE_OBJECT(Network_Display_Item, Network_Display_Item*  Network_Display_Item__Ptr = new Network_Display_Item);


    Network_Display_Item__Ptr->Network_Item_Type =   Network_Display_Item__Ptr->Catalytic_Transformation;
    Network_Display_Item__Ptr->Reactant1_Node_Index =  Reactant1_Node_Index;

    Network_Display_Item__Ptr->Reactant1_Node_Index =  Reactant2_Node_Index;
    Network_Display_Item__Ptr->Reactant2_Node_Index =  Catalyst_Node_Index;
    Network_Display_Item__Ptr->Product1_Node_Index =  Product1_Node_Index;
    Network_Display_Item__Ptr->Product1_Node_Index =  Product2_Node_Index;    
    Network_Display_Item__Ptr->Connection_Strength =  Connection_Strength;
    Network_Display_Item__Ptr->Conn_Selection_State = Conn_Selection_State;

    List_Of_Network_Display_Items.push_back(Network_Display_Item__Ptr);
}
//----------------------------------------------------------------------------------------------------------
int Network_Painter::Node_Col_For_Screen_Location(const Screen_Location*  const Screen_Location__Ptr) const

//  Return the Node Index for the specified Screen Location

{
    return(Screen_Location__Ptr->X / Display_Properties.Distance_In_Pixels_Between_Nodes);
}
//----------------------------------------------------------------------------------------------------------
int  Network_Painter::Node_Index_For_Screen_Location(const Screen_Location*  const Screen_Location__Ptr) const

//  Return the Node Index for the specified Screen Location

{
    int Node_Row =  Screen_Location__Ptr->Y / Display_Properties.Distance_In_Pixels_Between_Nodes;
    int Node_Col =  Screen_Location__Ptr->X / Display_Properties.Distance_In_Pixels_Between_Nodes;


    return (Node_Col + (Node_Row * Display_Properties.Num_Of_Nodes_Per_Display_Row));
}
//----------------------------------------------------------------------------------------------------------

bool Network_Painter::Screen_Location_Is_Valid_Node(const Screen_Location*  const Screen_Location__Ptr,
                                                    int  Number_Of_Nodes) const


    
//  Check whether the specified screen location corresponds to a valid Node in the Network

{    

    if ((Screen_Location__Ptr->X < 0) || (Screen_Location__Ptr->Y < 0))
    THEN
        return false;
    ELSEIF((Screen_Location__Ptr->X / Display_Properties.Distance_In_Pixels_Between_Nodes)
            >
        (Display_Properties.Num_Of_Nodes_Per_Display_Row - 1))
    THEN
        return false;
    ELSEIF  ((Node_Index_For_Screen_Location(Screen_Location__Ptr))
                >
            (Number_Of_Nodes -1))
    THEN
        return false;
    ELSE
        return true;
    ENDIF
}    
//----------------------------------------------------------------------------------------------------------
void Network_Painter::Zoom_In()


//  Zoom In the PaintBox display

{
    Display_Properties.Conn_Width_To_Strength_Ratio = Display_Properties.Conn_Width_To_Strength_Ratio*1.5;
    Display_Properties.Node_Width_To_Strength_Ratio = Display_Properties.Node_Width_To_Strength_Ratio*1.5;
    Display_Properties.Distance_In_Pixels_Between_Nodes =
        static_cast<int> (Display_Properties.Distance_In_Pixels_Between_Nodes*1.5);

    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();


}
//----------------------------------------------------------------------------------------------------------

void Network_Painter::Zoom_Out()

//  Zoom Out the PaintBox display

{
    Display_Properties.Conn_Width_To_Strength_Ratio = Display_Properties.Conn_Width_To_Strength_Ratio/1.5;
    Display_Properties.Node_Width_To_Strength_Ratio = Display_Properties.Node_Width_To_Strength_Ratio/1.5;
    Display_Properties.Distance_In_Pixels_Between_Nodes
        = static_cast<int> (Display_Properties.Distance_In_Pixels_Between_Nodes/1.5);

//    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();
}
//------------------------------------------------------------------------------
void Network_Painter::Redraw_Network()

//  Redraw the Network on the working Bitmap using the list of Network_Display_Items
//  And the list of Network Text Items

{

    unsigned int Item_Counter;
    Network_Display_Item*   Network_Display_Item__Ptr;

    for (Item_Counter = 0;
         Item_Counter < List_Of_Network_Display_Items.size();
         Item_Counter++)

    BEGIN

        Network_Display_Item__Ptr
            = (Network_Display_Item*)
                    List_Of_Network_Display_Items[Item_Counter];

        switch  (Network_Display_Item__Ptr->Network_Item_Type)

        FIRSTCASE  Network_Display_Item::Construction :

                Draw_Construction
                        (Network_Display_Item__Ptr->Reactant1_Node_Index,
                        Network_Display_Item__Ptr->Reactant2_Node_Index,
                        Network_Display_Item__Ptr->Product1_Node_Index,
                        Network_Display_Item__Ptr->Connection_Strength,
                        Network_Display_Item__Ptr->Conn_Selection_State,
                        false);  // Don't create a new Network_Display_Item

        NEXTCASE   Network_Display_Item::Transformation :

                Draw_Transformation
                        (Network_Display_Item__Ptr->Reactant1_Node_Index,
                         Network_Display_Item__Ptr->Product1_Node_Index,
                         Network_Display_Item__Ptr->Connection_Strength,
                         Network_Display_Item__Ptr->Conn_Selection_State,
                         false);   // Don't create a new Network_Display_Item

        NEXTCASE   Network_Display_Item::Molecule_Type :


            Draw_Molecule_Type(Network_Display_Item__Ptr->Node_Index,

                               Network_Display_Item__Ptr->Node_Strength,

                               Network_Display_Item__Ptr->Selection_State,
                               false); // Don't create a new Network_Display_Item
        DEFAULT :
        {}
        ENDSWITCH
    END


//  Now draw the Network Text Items

    list<Network_Text_Item*>::iterator  Iter;
    for (Iter = List_Of_Network_Text_Items.begin();
         Iter != List_Of_Network_Text_Items.end();
         Iter++)
    BEGIN
        Draw_Text_Output((*Iter)->XPos, (*Iter)->YPos, (*Iter)->Text_Item.c_str(), false);
    END
}
//----------------------------------------------------------------------------------------------------------

void    Network_Painter::Control_Unselected_Conn_Display(bool  Flag_Value)

//  Switch display of Unselected Connections on or off, depending on Flag_Value

{
    Display_Properties.Display_Unselected_Conns_Flag = Flag_Value;
    Clear_Network_Bitmap();
    Redraw_Network();
    Display_Network_Image();
}

//----------------------------------------------------------------------------------------------------------
void   Network_Painter::Set_Node_Selection_Colour_Map(Enum_Node_Selection_State  Selection_Type, 
                                                      Colour_Id Selection_Colour)

//  Define the display colour for a particular Node Selection Type

{
    Display_Properties.Node_Sel_State_To_Colour_Map[Selection_Type] = Selection_Colour;
}

//----------------------------------------------------------------------------------------------------------

void Network_Painter::Set_Number_Of_Nodes(int Num_Nodes)

//  Set the number of nodes for the display, 
//  and adjust the display to fit the available space

{
    TRACE("Network_Painter::Set_Number_Of_Nodes");
    int Nodes_Per_Display_Row = static_cast <int> (sqrt(double(Num_Nodes)));
    Nodes_Per_Display_Row = max(Nodes_Per_Display_Row, 1);

    int  Width;
    int  Height;
    GUI::Get_Size(Network_Drawing_Area_Id, Width, Height);

    int  Square_Side = min(Width, Height);
    int  Pixels_Per_Node = Square_Side / Nodes_Per_Display_Row;

    Display_Properties.Distance_In_Pixels_Between_Nodes = Pixels_Per_Node;
    Display_Properties.Num_Of_Nodes_Per_Display_Row =  Nodes_Per_Display_Row;
}
//----------------------------------------------------------------------------------------------------------
 


void Network_Painter::Clear_Network_Image()


//  Clear the working Bitmap used to build up the network display
//  In addition, clear the list of Network_Display_Items

{

    unsigned int Item_Counter;
    Network_Display_Item*   Network_Display_Item__Ptr;

    Clear_Network_Bitmap();

//  Clear the stored Network Display Items

    for (Item_Counter = 0;
         Item_Counter < List_Of_Network_Display_Items.size();
         Item_Counter++)
    BEGIN

        Network_Display_Item__Ptr
            = (Network_Display_Item*)
                    List_Of_Network_Display_Items[Item_Counter];
        DELETE_OBJECT(Network_Display_Item, Network_Display_Item__Ptr);
    END

//  Clear the list of pointers to Network Display Items

    List_Of_Network_Display_Items.clear();

//  Clear the stored network text items

    list<Network_Text_Item*>::iterator  Iter;
    for (Iter = List_Of_Network_Text_Items.begin();
         Iter != List_Of_Network_Text_Items.end();
         Iter++)
    BEGIN
        DELETE_OBJECT(Network_Text_Item, *Iter);
    END

//  Clear the list of pointers to Network Text Items
    
    List_Of_Network_Text_Items.clear();

//    GUI_Presentation_API::Clear(Widget_Ids::Chemistry_Draw_Area);
}
//------------------------------------------------------------------------------
void Network_Painter::Display_Network_Image()

//  Display the network image from the Bitmap on the PaintBox

{
//    TBitmap__Network_Display_Ptr->Canvas()->Pen()->Set_Color(clWhite());
//    TBitmap__Network_Display_Ptr->Canvas()->Brush()->Set_Color(clWhite());
//    TPaintBox__Network_Display_Ptr->Canvas()->Draw(0,0,TBitmap__Network_Display_Ptr);

    GUI_Presentation_API::Draw_Pixmap(Network_Drawing_Area_Id);
}
//------------------------------------------------------------------------------
void Network_Painter::Clear_Network_Bitmap()

//  Clear the working Bitmap used to build up the network display

{ 
    GUI::Clear(Network_Drawing_Area_Id);
}
//-----------------------------------------------------------------------------------------------------------

void Network_Painter::Support::Determine_Connection_Width_And_Colour(
                                int Conn_Selection_State,
                                double Connection_Strength,

                                int&   Line_Width,
                                Connection_Display_Colours* Colours,
                                const Network_Painter *const  Network_Painter_Ptr)

//  Set the main connection display colour and width, taking account of whether



//  connection is selected, and whether unselected connections are to be displayed

{
    const Network_Display_Properties*const
        Disp_Properties_Ptr =  &(Network_Painter_Ptr->Display_Properties);
        
    if (Conn_Selection_State == EOpt_Connection_Selected)
    THEN
        Colours->Main_Colour = Disp_Properties_Ptr->Main_Display_Colour;
        Colours->Reactant_End_Colour = Disp_Properties_Ptr->Reactant_End_Colour;
        Colours->Product_End_Colour = Disp_Properties_Ptr->Product_End_Colour;
        Colours->Catalyst_End_Colour = Disp_Properties_Ptr->Catalyst_End_Colour;
        if (Connection_Strength <= 0)
        THEN
            Line_Width = 0;
        ELSE
            Line_Width = static_cast<int> (2 * log10(Connection_Strength * Disp_Properties_Ptr->Conn_Width_To_Strength_Ratio));
        ENDIF
    ELSEIF (Conn_Selection_State == EOpt_Connection_Not_Selected)
    THEN
        if (Disp_Properties_Ptr->Display_Unselected_Conns_Flag)
        THEN
            Colours->Main_Colour = Disp_Properties_Ptr->Main_Display_Colour;
            Colours->Reactant_End_Colour = Disp_Properties_Ptr->Reactant_End_Colour;
            Colours->Catalyst_End_Colour = Disp_Properties_Ptr->Catalyst_End_Colour;
            Colours->Product_End_Colour = Disp_Properties_Ptr->Product_End_Colour;
                        
            if (Connection_Strength <= 0)
            THEN
                Line_Width = 0;
            ELSE
                Line_Width = static_cast<int> (log10(Connection_Strength * Disp_Properties_Ptr->Conn_Width_To_Strength_Ratio));
            ENDIF
        ELSE
            Line_Width = 0;
        ENDIF

    ENDIF
}

//-----------------------------------------------------------------------------------------------------------

void Network_Painter::Support::Join_Nodes(const int  Start_Node_Index,
                                          const int  End_Node_Index,
                                          const int   Line_Width,
                                          const Colour_Id Main_Disp_Colour,
                                          const Colour_Id Start_Disp_Colour,
                                          const Colour_Id End_Disp_Colour, 
                                          const Network_Painter *const  Network_Painter_Ptr,
                                          Widget_Ids::Drawing_Area_Id Drawing_Area_Id)

//  Draw a line Joining two nodes in the Network. 

{      
    Screen_Location Connection_Start =  Get_Screen_Location_For_Node(Start_Node_Index,Network_Painter_Ptr);
    Screen_Location  Connection_End  =  Get_Screen_Location_For_Node(End_Node_Index,Network_Painter_Ptr);


    Join_Screen_Locations(Connection_Start,
                          Connection_End,
                          Line_Width,
                          Main_Disp_Colour,
                          Start_Disp_Colour,
                          End_Disp_Colour,
                          Drawing_Area_Id);
}
//-----------------------------------------------------------------------------------------------------------
void Network_Painter::Support::Join_Screen_Locations(const Screen_Location Start_Screen_Loc,
                                          const Screen_Location End_Screen_Loc,
                                          const int  Line_Width,
                                          const Colour_Id Main_Disp_Colour,
                                          const Colour_Id Start_Disp_Colour,
                                          const Colour_Id End_Disp_Colour,
                                          Widget_Ids::Drawing_Area_Id Network_Drawing_Area_Id)

//  Draw a line joining the specified Screen Locations
//  Draw ends with different colours

{
    int End_Segment_X =  static_cast<int>(
                         (0.8 * Start_Screen_Loc.X) + (0.2 * End_Screen_Loc.X));

    int End_Segment_Y =  static_cast<int>( 
                         (0.8 * Start_Screen_Loc.Y) + (0.2 * End_Screen_Loc.Y));

    GUI_Presentation_API::Set_Foreground_Colour(Network_Drawing_Area_Id, 
                                                Colour_Ids::Red);

    GUI_Presentation_API::Draw_Line(Network_Drawing_Area_Id,
                                         Start_Screen_Loc.X,
                                         Start_Screen_Loc.Y,
                                         End_Segment_X,
                                         End_Segment_Y);
//  Middle part of line

    int Start_Segment_X = End_Segment_X;
    int Start_Segment_Y = End_Segment_Y;

    End_Segment_X =   static_cast<int>(
                      (0.2 * Start_Screen_Loc.X) + (0.8 * End_Screen_Loc.X));
    End_Segment_Y =   static_cast<int>(
                      (0.2 * Start_Screen_Loc.Y) + (0.8 * End_Screen_Loc.Y));


    GUI_Presentation_API::Set_Foreground_Colour(Network_Drawing_Area_Id, 
                                                Colour_Ids::Black);

    GUI_Presentation_API::Draw_Line(Network_Drawing_Area_Id,
                                         Start_Segment_X,
                                         Start_Segment_Y,
                                         End_Segment_X,
                                         End_Segment_Y);

   
//  End part of line

    GUI_Presentation_API::Set_Foreground_Colour(Network_Drawing_Area_Id, 
                                                Colour_Ids::Blue);

    
    GUI_Presentation_API::Draw_Line(Network_Drawing_Area_Id,
                                         End_Segment_X,
                                         End_Segment_Y,
                                         End_Screen_Loc.X,
                                         End_Screen_Loc.Y);
}
//-----------------------------------------------------------------------------------------------------------
void Network_Painter::Support::Join_Node_To_Mid_Point
                                          (const int  Join_Node_Index,
                                           const int  Start_Node_Index,
                                           const int  End_Node_Index,
                                           const int  Line_Width,
                                           const Colour_Id Main_Disp_Colour,
                                           const Colour_Id Start_Disp_Colour,
                                           const Colour_Id End_Disp_Colour,
                                           Widget_Ids::Drawing_Area_Id Drawing_Area_Id,
                                           const Network_Painter *const  Network_Painter_Ptr)
{
    Assert (End_Disp_Colour == 0,"Network_Painter::Support::Join_Node_To_Mid_Point - End_Display_Colour is not zero");
    Screen_Location  Screen_Loc_Mid_Point;
    Screen_Loc_Mid_Point = Get_Screen_Location_For_Mid_Point
                                                (Start_Node_Index,
                                                 End_Node_Index,

                                                 Network_Painter_Ptr);

    Screen_Location  Screen_Loc_Join_Node;
    Screen_Loc_Join_Node = Get_Screen_Location_For_Node(Join_Node_Index,
                                                        Network_Painter_Ptr);

    Join_Screen_Locations(Screen_Loc_Join_Node,
                          Screen_Loc_Mid_Point,
                          Line_Width,
                          Main_Disp_Colour,
                          Start_Disp_Colour,
                          Main_Disp_Colour,   // End colour is same as middle part of line
                          Drawing_Area_Id);
}
//-----------------------------------------------------------------------------------------------------------
Network_Painter_Prot::Screen_Location
        Network_Painter::Support::Get_Screen_Location_For_Mid_Point
                                        (int Start_Node_Index,
                                         int End_Node_Index,
                                         const Network_Painter *const  Network_Painter_Ptr)


                                         
//  Determine the Screen Location (in pixels) for the mid point between the
//  Start Node and End Node

{
    Screen_Location  Screen_Location__For_Start_Node // In Pixels

        = Get_Screen_Location_For_Node(Start_Node_Index, Network_Painter_Ptr);
                                         
    Screen_Location  Screen_Location__For_End_Node // In Pixels

        = Get_Screen_Location_For_Node(End_Node_Index, Network_Painter_Ptr);

    int Return_Value_X = (Screen_Location__For_Start_Node.X + Screen_Location__For_End_Node.X)/2;
    int Return_Value_Y = (Screen_Location__For_Start_Node.Y + Screen_Location__For_End_Node.Y)/2;

    Screen_Location  Screen_Loc_Mid_Point;
    Screen_Loc_Mid_Point.X = Return_Value_X;
    Screen_Loc_Mid_Point.Y = Return_Value_Y;

    return(Screen_Loc_Mid_Point);
}          

//-----------------------------------------------------------------------------------------------------------
Network_Painter_Prot::Screen_Location
        Network_Painter::Support::Get_Screen_Location_For_Node
                                        (int Node_Index,
                                         const Network_Painter *const  Network_Painter_Ptr)


//  Determine the Screen Location (in Pixels) associated with
//  the specified Node_Index

{
    int     Node_Row;
    int     Node_Display_X;   // X Co-ordinate for Node in pixels
    int     Node_Display_Y;   // Y Co-ordinate for Node in pixels
    Screen_Location  Screen_Location__For_Node; // In Pixels

    const Network_Display_Properties*const
    Display_Properties_Ptr =  &(Network_Painter_Ptr->Display_Properties);

    Node_Row = Node_Index / Display_Properties_Ptr->Num_Of_Nodes_Per_Display_Row;

    Node_Display_Y = Node_Row * Display_Properties_Ptr->Distance_In_Pixels_Between_Nodes;
    Node_Display_X = (Node_Index - (Node_Row * Display_Properties_Ptr->Num_Of_Nodes_Per_Display_Row)
                               ) * Display_Properties_Ptr->Distance_In_Pixels_Between_Nodes;

    Node_Display_Y = Node_Display_Y + (Display_Properties_Ptr->Distance_In_Pixels_Between_Nodes/2);
                                                        // Allow a border

    Node_Display_X = Node_Display_X + (Display_Properties_Ptr->Distance_In_Pixels_Between_Nodes/2);

    Screen_Location__For_Node.X = Node_Display_X;
    Screen_Location__For_Node.Y = Node_Display_Y;

    return(Screen_Location__For_Node);
}





