/****************************************************************************
*
*	  Header Filename:  Network_Painter.h
*	  Module Name:      Network_Painter
*	  Description:
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	             Update Description
*		=======    ===========       ==================
*
*
*
*
****************************************************************************/

#ifndef Network_Painter_H
#define Network_Painter_H

//  Declaration of inherited class(es)

#include    "Network_Painter_Prot.h"

//----------------------------------------------------------------------------------------------------------

class   Network_Painter: public Network_Painter_Prot

//  Provides functionality for display and animation of Networks
//  consisting of Nodes and Connections

{
    class Support;
    friend class Support;

private:

    struct  Network_Text_Item

//      Stores text for output with the network

        {
            int     XPos;
            int     YPos;
            string  Text_Item;
        };

    struct    Network_Display_Item

//      Stores an item for display as part of the network

        {
            enum   {Construction,
                    Fission,
                    Recombination,
                    Transformation,
                    Catalytic_Construction,
                    Catalytic_Fission,
                    Catalytic_Recombination,
                    Catalytic_Transformation,
                    Molecule_Type} Network_Item_Type;

            int     Reactant1_Node_Index;
            int     Reactant2_Node_Index;
            int     Product1_Node_Index;
            int     Product2_Node_Index;
            int     Catalyst_Node_Index;
            double   Connection_Strength;
            int     Node_Index;
            double   Node_Strength;
 
            Enum_Node_Selection_State  Selection_State;
            int Conn_Selection_State;
        } ;

private:
//===== Relationships with other classes ===================================================================
    vector<Network_Display_Item*> List_Of_Network_Display_Items;
    list<Network_Text_Item*>    List_Of_Network_Text_Items;

//    TBitmap* TBitmap__Network_Display_Ptr;
                                        // Bitmap used by the
                                        // Network_Painter to draw the
                                        // Network

//    TPaintBox* TPaintBox__Network_Display_Ptr;
                                        // PaintBox on which the
                                        // Network_Painter will paint

    Widget_Ids::Drawing_Area_Id Network_Drawing_Area_Id;

//  Network_Display_Properties - A composite class. Used to hold the
//                               Network_Painter's Display_Properties
                                        
//==========================================================================================================

//      Private Class Data

private:
    class Network_Display_Properties
    {
     public:
        double   Conn_Width_To_Strength_Ratio;
        double   Node_Width_To_Strength_Ratio;
        int     Num_Of_Nodes_Per_Display_Row;
        int     Distance_In_Pixels_Between_Nodes;
        bool    Display_Unselected_Conns_Flag;
        Colour_Id  Main_Display_Colour;
        Colour_Id  Reactant_End_Colour;
        Colour_Id  Product_End_Colour;
        Colour_Id  Catalyst_End_Colour;
        map <Enum_Node_Selection_State,Colour_Id> Node_Sel_State_To_Colour_Map;   //  First item is
    };

    Network_Display_Properties  Display_Properties;
//    TRect      TRect__Network_Display_Rectangle;
    int     PaintBox_Top_Initial; // Initial value. Neede due to Lazarus bug 361

public:
    void Clear();
    void Draw_Construction(int Reactant1_Node_Index,
                              int Reactant2_Node_Index,
                              int Product1_Node_Index,
                              double Connection_Strength,
                              int Conn_Selection_State,
                              bool Bool__Store_Network_Item = true);
    void Draw_Fission     (int Reactant1_Node_Index,
                           int Product1_Node_Index,
                           int Product2_Node_Index,
                           double Connection_Strength,
                           int Conn_Selection_State,
                           bool Bool__Store_Network_Item = true);
                              
    void Draw_Transformation (int Reactant1_Node_Index,
                              int Product1_Node_Index,
                              double Connection_Strength,
                              int Conn_Selection_State,
                              bool Bool__Store_Network_Item = true);

    void Draw_Molecule_Type(const int Molecule_Type_Identifier,
                            const double Node_Strength,




                            const Enum_Node_Selection_State  Selection_State,
                            const bool Bool__Store_Network_Item = true);

    void    Draw_Text_Output(int X_Pos, int Y_Pos,const  char* Output_Text_Ptr,
                             bool Bool__Store_Network_Item = true);
     
                             
    Network_Painter(int     NumOfNodesPerDisplayRow,
                    int     SpaceInPixelsBetweenNodes,
                    double   Typical_Connection_Strength,
                    double   Typical_Node_Strength,
            //        void*   TPaintBox__NetworkDisplayPtr,
                    bool    Display_All_Connections_Ind,
                    Widget_Ids::Drawing_Area_Id Draw_Area_Id);
    ~Network_Painter(){}; 

    int Node_Index_For_Screen_Location(const Screen_Location*  const Screen_Location__Ptr) const;
    int Node_Col_For_Screen_Location(const Screen_Location*  const Screen_Location__Ptr) const;
    bool Screen_Location_Is_Valid_Node(const Screen_Location*  const Screen_Location__Ptr,
                                       int  Number_Of_nodes) const;
            
//      Private Functions
private:
    void Clear_Network_Bitmap();
    void Clear_Network_Image();
    void Display_Network_Image();
    void Redraw_Network();
    void Zoom_In();
    void Zoom_Out();
    void Increase_Connection_Width();
    void Decrease_Connection_Width();
    void Increase_Node_Width();
    void Decrease_Node_Width();
    void Control_Unselected_Conn_Display(bool  Flag_Value);
//    TBitmap* Get_TBitmap_Network_Display_Ptr() const;
    void Set_Node_Selection_Colour_Map(Enum_Node_Selection_State  Selection_Type, 
                                       Colour_Id  Selection_Colour);
    void Make_And_Store_Connection_Display_Item(const double&   Connection_Strength,
                                                const int      Conn_Selection_State,
                                                const int&     Reactant1_Node_Index,
                                                const int&     Reactant2_Node_Index,
                                                const int&     Product1_Node_Index,
                                                const int&     Product2_Node_Index,
                                                const int&     Catalyst_Node_Index);

    void Set_Number_Of_Nodes(int Num_Nodes);
        
    private:
    class Support
    {
    public:

        struct    Connection_Display_Colours

        {
            Colour_Id   Main_Colour;
            Colour_Id   Reactant_End_Colour;
            Colour_Id   Product_End_Colour;
            Colour_Id   Catalyst_End_Colour;
        };
            
        static void  Determine_Connection_Width_And_Colour(
                                int   Conn_Selection_State,
                                double Connection_Strength,

                                int&   Line_Width,
                                Connection_Display_Colours* Colours,
                                const Network_Painter *const  Network_Painter_Ptr);


        static Network_Painter_Prot::Screen_Location Get_Screen_Location_For_Mid_Point
                                            (int Start_Node_Index,
                                             int End_Node_Index,
                                             const Network_Painter *const  Network_Painter_Ptr);
                                             
        static Network_Painter_Prot::Screen_Location Get_Screen_Location_For_Node
                                            (int Node_Index,
                                            const Network_Painter *const  Network_Painter_Ptr);

                                            
        static void Join_Nodes(const int  Start_Node_Index,
                               const int  End_Node_Index,
                               const int  Line_Width,
                               const Colour_Id Main_Disp_Colour,

                               const Colour_Id Start_Disp_Colour,
                               const Colour_Id End_Disp_Colour,                             
                               const Network_Painter *const  Network_Painter_Ptr,
                               Widget_Ids::Drawing_Area_Id Drawing_Area_Id);


        static void Join_Node_To_Mid_Point(const int  Join_Node_Index,
                                           const int  Start_Node_Index,
                                           const int  End_Node_Index,
                                           const int  Line_Width,
                                           const Colour_Id Main_Disp_Colour,
                                           const Colour_Id Second_Disp_Colour,
                                           const Colour_Id Third_Disp_Colour,
                                           Widget_Ids::Drawing_Area_Id Network_Drawing_Area_Id,
                                           const Network_Painter *const  Network_Painter_Ptr);


        static void Join_Screen_Locations(const Screen_Location Start_Screen_Loc,
                                          const Screen_Location End_Screen_Loc,
                                          const int  Line_Width,
                                          const Colour_Id Main_Disp_Colour,
                                          const Colour_Id Start_Disp_Colour,
                                          const Colour_Id End_Disp_Colour,
                                          Widget_Ids::Drawing_Area_Id Network_Drawing_Area_Id);
    };

};

#endif  //Network_Painter_H
