/****************************************************************************
*
*	  Module Filename:  System_Monitor.cpp
*	  Module Name:      System Monitor
*	  Description:      Built in system monitor. Mainly for diagnostic purposes
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*     Version    Date                       Update Description
*     =======    ===========                ==================
*
*
*
*
****************************************************************************/

#include  "globldef.h"
#include  "Library_Headers.h"

System_Monitor*  System_Monitor::Instance_Ptr = 0;
//==============================================================================================================

string System_Monitor::Get_Current_Class()

//  Get the name of the class to which the System_Monitor's iterator currently points

{
    return Object_Statistics_Iter->first;  
}
//---------------------------------------------------------------------------------------------------------------

int System_Monitor::Get_Num_Creates_Current_Class()

//  Get the number of object creations that have been recorded for the 
//  class to which the System_Monitor's iterator currently points

{
    return Object_Statistics_Iter->second.first;  
}
//---------------------------------------------------------------------------------------------------------------

int System_Monitor::Get_Num_Deletes_Current_Class()

//  Get the number of object deletions that have been recorded for the 
//  class to which the System_Monitor's iterator currently points

{
    return Object_Statistics_Iter->second.second;  
}
//---------------------------------------------------------------------------------------------------------------

int System_Monitor::Get_Num_Objects_Current_Class()

//  Get the number of object instances (difference between recorced creations and deletes) for the 
//  class to which the System_Monitor's iterator currently points

{
    return (Object_Statistics_Iter->second.first - Object_Statistics_Iter->second.second);  
}
//---------------------------------------------------------------------------------------------------------------

bool System_Monitor::Iter_To_First_Class()

//  Set the System_Monitor's iterator to point to the first Class for which
//  object statistics are held. Return false if no statistics

{
    Object_Statistics_Iter = Object_Statistics.begin();
    if (Object_Statistics_Iter not_eq Object_Statistics.end())
    THEN
        return true;
    ELSE
        return false;
    END
}   
//-------------------------------------------------------------------------------------------------------------- 

bool    System_Monitor::Iter_To_Next_Class()

//  Move the System_Monitor's iterator to point to the next Class for which
//  object statistics are held.

{
    if(++Object_Statistics_Iter not_eq Object_Statistics.end())
    THEN
        return true;

    ELSE
        return false;
    ENDIF
}
//-------------------------------------------------------------------------------------------------------------- 

System_Monitor*  System_Monitor::Get_Ptr()

//  Get a pointer to the System_Monitor. Tis a fatal error is the System_Monitor has not yet been 
//  initialised.

{
    Assert(Instance_Ptr not_eq 0,"System_Monitor::Get_Ptr: Called before an instance has been created");
    return (Instance_Ptr);
}
//-------------------------------------------------------------------------------------------------------------- 

void  System_Monitor::Make()

//  Initialise the single instance System_Monitor. Its a fatal error if one already exists.

{
    Assert(Instance_Ptr == 0,"System_Monitor::Make -  Invalid call; System_Monitor has already been created");    
    CREATE_OBJECT(System_Monitor,Instance_Ptr = new System_Monitor);
}
//-------------------------------------------------------------------------------------------------------------- 

void  System_Monitor::Object_Creation(string Class_Name)

//  Increment the object creation counter for the specified class. 
//  If this is the first object created, then the class name will not exist in the map, 
//  and so it is inserted with the default value for the element pair (that's
//  the normal behaviour for an STL map, so no special code here!)

{
    Object_Statistics[Class_Name].first++;
}
//-------------------------------------------------------------------------------------------------------------------------

void  System_Monitor::Object_Deletion(string Class_Name)

//  Increment the object deletion counter for the specified class. 

{
    Object_Statistics[Class_Name].second++; 
}











    
